@@ -2,6 +2,7 @@ package config
 
 import (
 	"bufio"
+	"errors"
 	"fmt"
 	"io/ioutil"
 	"net"
@@ -13,6 +14,8 @@ import (
 	"github.com/ghodss/yaml"
 	"github.com/sirupsen/logrus"
 	v1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/tools/clientcmd"
 
 	"github.com/openshift/baremetal-runtimecfg/pkg/utils"
@@ -26,13 +29,16 @@ type Cluster struct {
 	Domain                 string
 	APIVIP                 string
 	APIVirtualRouterID     uint8
+	APIVIPRecordType       string
+	APIVIPEmptyType        string
 	DNSVIP                 string
 	DNSVirtualRouterID     uint8
 	IngressVIP             string
 	IngressVirtualRouterID uint8
+	IngressVIPRecordType   string
+	IngressVIPEmptyType    string
 	VIPNetmask             int
 	MasterAmount           int64
-	EtcdBackends           string
 }
 
 type Backend struct {
@@ -201,13 +207,25 @@ func GetConfig(kubeconfigPath, clusterConfigPath, resolvConfPath string, apiVip
 	}
 
 	vips := make([]net.IP, 0)
+	node.Cluster.APIVIPRecordType = "A"
+	node.Cluster.APIVIPEmptyType = "AAAA"
 	if apiVip != nil {
 		vips = append(vips, apiVip)
 		node.Cluster.APIVIP = apiVip.String()
+		if apiVip.To4() == nil {
+			node.Cluster.APIVIPRecordType = "AAAA"
+			node.Cluster.APIVIPEmptyType = "A"
+		}
 	}
+	node.Cluster.IngressVIPRecordType = "A"
+	node.Cluster.IngressVIPEmptyType = "AAAA"
 	if ingressVip != nil {
 		vips = append(vips, ingressVip)
 		node.Cluster.IngressVIP = ingressVip.String()
+		if ingressVip.To4() == nil {
+			node.Cluster.IngressVIPRecordType = "AAAA"
+			node.Cluster.IngressVIPEmptyType = "A"
+		}
 	}
 	if dnsVip != nil {
 		vips = append(vips, dnsVip)
@@ -230,57 +248,66 @@ func GetConfig(kubeconfigPath, clusterConfigPath, resolvConfPath string, apiVip
 			node.DNSUpstreams = append(node.DNSUpstreams, upstream)
 		}
 	}
+	// If we end up with no upstream DNS servers we'll generate an invalid
+	// coredns config. Error out so the init container retries.
+	if len(node.DNSUpstreams) < 1 {
+		return node, errors.New("No upstream DNS servers found")
+	}
 
 	prefix, _ := nonVipAddr.Mask.Size()
 	node.Cluster.VIPNetmask = prefix
 	node.VRRPInterface = vipIface.Name
 
-	domain := fmt.Sprintf("%s.%s", clusterName, clusterDomain)
-	node.LBConfig, err = GetLBConfig(domain, apiPort, lbPort, statPort, apiVip)
-	if err != nil {
-		return node, err
-	}
-	etcdBackends, err := getSortedBackends(domain)
-	if err != nil {
-		return node, err
-	}
-	for _, backend := range etcdBackends {
-		if len(node.Cluster.EtcdBackends) > 0 {
-			node.Cluster.EtcdBackends += ","
-		}
-		node.Cluster.EtcdBackends += fmt.Sprintf("https://%s:2379", strings.TrimRight(backend.Host, "."))
-	}
-
 	return node, err
 }
 
-func getSortedBackends(domain string) (backends []Backend, err error) {
-	srvs, err := utils.GetEtcdSRVMembers(domain)
+func getSortedBackends(kubeconfigPath string) (backends []Backend, err error) {
+
+	config, err := clientcmd.BuildConfigFromFlags("", kubeconfigPath)
 	if err != nil {
 		log.WithFields(logrus.Fields{
 			"err": err,
-		}).Info("Failed to get Etcd SRV members")
-		srvs = []*net.SRV{}
-		err = nil
+		}).Info("Failed to get client config")
+		return []Backend{}, err
+	}
+	clientset, err := kubernetes.NewForConfig(config)
+	if err != nil {
+		log.WithFields(logrus.Fields{
+			"err": err,
+		}).Info("Failed to get client")
+		return []Backend{}, err
+	}
+	nodes, err := clientset.CoreV1().Nodes().List(metav1.ListOptions{
+		LabelSelector: "node-role.kubernetes.io/master=",
+	})
+	if err != nil {
+		log.WithFields(logrus.Fields{
+			"err": err,
+		}).Info("Failed to get master Nodes list")
+		return []Backend{}, err
+	}
+	for _, node := range nodes.Items {
+		masterIp := ""
+		for _, address := range node.Status.Addresses {
+			if address.Type == v1.NodeInternalIP {
+				masterIp = address.Address
+				break
+			}
+		}
+		if masterIp != "" {
+			backends = append(backends, Backend{Host: node.ObjectMeta.Name, Address: masterIp})
+		} else {
+			log.Warnf("Could not retrieve node's IP for %s", node.ObjectMeta.Name)
+		}
 	}
 
-	for _, srv := range srvs {
-		addr, err := utils.GetFirstAddr(srv.Target)
-		if err != nil {
-			log.WithFields(logrus.Fields{
-				"member": srv.Target,
-			}).Error("Failed to get address for member")
-			continue
-		}
-		backends = append(backends, Backend{Host: srv.Target, Address: addr, Port: srv.Port})
-	}
 	sort.Slice(backends, func(i, j int) bool {
 		return backends[i].Address < backends[j].Address
 	})
 	return backends, err
 }
 
-func GetLBConfig(domain string, apiPort, lbPort, statPort uint16, apiVip net.IP) (ApiLBConfig, error) {
+func GetLBConfig(kubeconfigPath string, apiPort, lbPort, statPort uint16, apiVip net.IP) (ApiLBConfig, error) {
 	config := ApiLBConfig{
 		ApiPort:  apiPort,
 		LbPort:   lbPort,
@@ -291,11 +318,11 @@ func GetLBConfig(domain string, apiPort, lbPort, statPort uint16, apiVip net.IP)
 		config.FrontendAddr = "::"
 	}
 
-	backends, err := getSortedBackends(domain)
+	backends, err := getSortedBackends(kubeconfigPath)
 	if err != nil {
 		log.WithFields(logrus.Fields{
-			"domain": domain,
-		}).Error("Failed to retrieve API member information")
+			"kubeconfigPath": kubeconfigPath,
+		}).Error("Failed to retrieve API members information")
 		return config, err
 	}
 
